@@ -8,6 +8,7 @@ from typing import Type, Tuple, Sequence, Dict, Callable, Any, Optional, List
 from django.apps import apps
 from django.conf.urls import url
 from django.db.models import QuerySet, Model
+from rest_framework import status
 from rest_framework.authentication import BaseAuthentication
 from rest_framework.parsers import FormParser, MultiPartParser
 from rest_framework.permissions import BasePermission
@@ -63,6 +64,7 @@ class JsonApiModelViewBuilder:
         self._after_update_callback = None
         self._before_delete_callback = None
         self._after_delete_callback = None
+        self._before_list_callback = None
         self._after_list_callback = None
         if queryset is None:
             self._queryset = self._model.objects
@@ -215,6 +217,12 @@ class JsonApiModelViewBuilder:
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_DELETE)
         return self
 
+    def before_list(self,
+                    before_list_callback: Callable[[Request, QuerySet], QuerySet] = None) -> 'JsonApiModelViewBuilder':
+        self._before_list_callback = before_list_callback
+        self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
+        return self
+
     def after_list(self, after_list_callback: Callable[[Any], Any] = None) -> 'JsonApiModelViewBuilder':
         self._after_list_callback = after_list_callback
         self.__warn_if_method_not_available(json_api_spec_http_methods.HTTP_GET)
@@ -298,7 +306,7 @@ class JsonApiModelViewBuilder:
         def perform_create(view, serializer):
             instance = serializer.save()
             if self._after_create_callback is not None:
-                self._after_create_callback(instance, serializer)
+                self._after_create_callback(view.request, instance, serializer)
 
         def perform_destroy(view, instance):
             if self._before_delete_callback is not None:
@@ -316,10 +324,22 @@ class JsonApiModelViewBuilder:
         def perform_update(view, serializer):
             instance = serializer.save()
             if self._after_update_callback is not None:
-                self._after_update_callback(instance, serializer)
+                self._after_update_callback(view.request, instance, serializer)
+
+        def perform_list(view, request, *args, **kwargs):
+            queryset = view.filter_queryset(view.get_queryset())
+
+            if self._before_list_callback is not None:
+                queryset = self._before_list_callback(request, queryset)
+
+            page = view.paginate_queryset(queryset)
+            if page is not None:
+                serializer = view.get_serializer(page, many=True)
+                response = view.get_paginated_response(serializer.data)
+            else:
+                serializer = view.get_serializer(queryset, many=True)
+                response = Response(serializer.data)
 
-        def perform_list(view, *args, **kwargs):
-            response = super(view.__class__, view).list(*args, **kwargs)
             if self._after_list_callback is not None:
                 response.data = self._after_list_callback(response.data)
             return response
@@ -569,15 +589,14 @@ class JsonApiResourceViewBuilder:
         if len(url_resource_name) == 0:
             url_resource_name = self._resource_name
 
-
         urls.extend([
             url(rf'^{urls_prefix}{url_resource_name}{urls_suffix}$',
                 get_view_set.as_view({'get': 'list', 'post': 'create'}, name=f'list_{self._resource_name}'),
-                name=f'list-{self._resource_name}-action'),
+                name=f'list-{self._resource_name}'),
             url(rf'^{urls_prefix}{url_resource_name}{urls_suffix}/(?P<{self._unique_identifier}>[^/.]+)/$',
                 patch_view_set.as_view({'get': 'get', 'patch': 'update', 'delete': 'destroy'},
-                                       name=f'patch_destroy_{self._resource_name}'),
-                name=f'patch-delete-{self._resource_name}-action')
+                                       name=f'get_{self._resource_name}'),
+                name=f'{self._resource_name}-detail')
         ])
         return urls
 
@@ -590,15 +609,19 @@ def json_api_view(resource_name: str,
                   permission_classes: Optional[Sequence[Type[BasePermission]]] = None,
                   authentication_classes: Optional[Sequence[Type[BaseAuthentication]]] = None,
                   urls_prefix: str = '',
-                  urls_suffix: str = '', ) -> FunctionType:
+                  urls_suffix: str = '',
+                  multiple_resource = True) -> FunctionType:
     def decorator(func: Callable[[Request], Tuple[Dict, int]]):
         builder = JsonApiResourceViewBuilder(action_name=resource_name,
                                              allowed_methods=[method],
                                              permission_classes=permission_classes,
                                              authentication_classes=authentication_classes)
-        if method == json_api_spec_http_methods.HTTP_GET:
+        if method == json_api_spec_http_methods.HTTP_GET and multiple_resource:
             return builder.on_list(list_callback=func) \
                 .get_urls(urls_prefix=urls_prefix, urls_suffix=urls_suffix)
+        elif method == json_api_spec_http_methods.HTTP_GET:
+            return builder.on_get(get_callback=func) \
+                .get_urls(urls_prefix=urls_prefix, urls_suffix=urls_suffix)
         elif method == json_api_spec_http_methods.HTTP_POST:
             return builder.on_create(create_callback=func) \
                 .get_urls(urls_prefix=urls_prefix, urls_suffix=urls_suffix)
